#include "phc.h"
#include <math.h>

PPC::PPC(float hfov, int _w, int _h) : w(_w), h(_h) {

  C = V3(0.0f, 0.0f, 0.0f);
  a = V3(1.0f, 0.0f, 0.0f);
  b = V3(0.0f, -1.0f, 0.0f);
  float tan2 = tanf(hfov/2.0f/180.0f*3.14159f);
  c = V3(-(float)w/2.0f, (float)h/2.0f, -(float)w/2.0f/tan2);

  UpdateInvCam();

}

void PPC::UpdateInvCam() {

  invcam.SetColumn(0, a);
  invcam.SetColumn(1, b);
  invcam.SetColumn(2, c);
  invcam = invcam.Invert();

}

bool PPC::Project(V3 P, V3& projP) {

  V3 q = invcam*(P-C);
  if (q[2] <= 0.0f)
    return false;

  projP[0] = q[0] / q[2];
  projP[1] = q[1] / q[2];
  projP[2] = 1.0f / q[2];
  return true;

}

V3 PPC::GetPoint(float uf, float vf) {

  return C+c+a*uf+b*vf;

}