#include "scene.h"
#include "v3.h"
#include "M3x3.h"

#include <iostream>


using namespace std;

/* additions

V3 multiplication and division by scalar
V3 cross product
M3x3 matrix inversion
M3x3 output to ostream operator (friend)
TMesh class with Load method
*/


Scene *scene = 0;

Scene::Scene() {

  gui = new GUI;
  fb = new FrameBuffer(20, 40, 640, 480);
  fb->label("Frame buffer");
  gui->show();
  tmesh = new TMesh();
  tmesh->Load("geometry/teapot57K.bin");
  V3 center = tmesh->GetCenter();
  V3 newCenter(0.0f, 0.0f, -100.0f);
  tmesh->Translate(newCenter-center);
  Show();

}

void Scene::Show() {

  gui->uiw->position(20+fb->w+20, 40);
  unsigned int color = 0xFF000000;
  fb->Set(color);
  fb->show();

}

void Scene::DBG() {

  {
    fb->Set(0xFFFFFFFF);
    PPC ppc(60.0f, fb->w, fb->h);
    tmesh->RenderPoints(&ppc, fb);
    fb->redraw();
    return;

    V3 v(3.0f, 0.0f, -10.0f);
    v = ppc.GetPoint(0.0f, 0.0f);
    V3 projV;
    if (ppc.Project(v, projV)) {
      fb->DrawDiagonalCross((int)projV[0],
        (int)projV[1], 0xFF000000, 17);
    }
    fb->redraw();
    return;
  }

  {
    TMesh tmesh;
    tmesh.Load("geometry/teapot1K.bin");
    return;
  }

  {

    M3x3 m;
    m[0] = V3(13.0f, -12.0f, 3.0f);
    m[1] = V3(-4.0f, 3.0f, 2.0f);
    m[2] = V3(5.0f, -12.0f, -10.0f);
    cerr << m << m.Invert() << m.Invert()*m << m*m.Invert();
    return;

  }

  {
    V3 v0(1.0f, 0.3f, 2.0f);
    V3 v1(1.0f, 0.3f, 2.0f);
    V3 v2(0.0f, 0.0f, 0.0f);
    v2 = v0+v1;
    cerr << v2 << endl;
    cerr << v2[0] << endl;
    v2[1] = 34.0f;
    cerr << v2[1] << endl;
    M3x3 m;
    m[0] = v0;
    m[1] = v1;
    m[2] = v2;

  }

  {
    unsigned int colors[2] = {0xFFAAAAAA, 0xFFFFFFFF};
    fb->SetGrid(40, colors);
    fb->redraw();
    return;
  }

  {
    int u, v = 50, sizePix = 15;
    unsigned int color = 0xFFFFFFFF;
    for (u = 0; u < fb->w; u++) {
      fb->Set(0xFF000000);
      fb->DrawDiagonalCross(u, v, color, sizePix);
      fb->redraw();
      Fl::check();
    }
  }

}

